// Dialog Tools
// (c) FiveWin IDE

#include "FiveWin.ch"
#include "Visual.ch"

#define SM_CYBORDER         6
#define SM_CYCAPTION        4

#define SIZE_RESTORED       0
#define SIZE_MINIMIZED      1
#define SIZE_MAXIMIZED      2

#define CLR_BTNFACE        15

extern WndTop

#ifdef __XPP__
   #define Super ::TForm
#endif

memvar oIDE

//----------------------------------------------------------------------------//

CLASS TDesigner FROM TForm

   DATA   oWndTools     // reference to the Tools window

   DATA   oEdit          // Associated GET multiline with source code
   DATA   cFileName     // FileName on disk for the form
   DATA   cPrgName      // FileName on disk for the source code
   DATA   cClassName INIT "TForm"  // New designed Class name

   DATA   lPaintGrid AS LOGICAL INIT .t.
   DATA   nGridX AS NUMERIC INIT 8
   DATA   nGridY AS NUMERIC INIT 8

   DATA   nBoxTop, nBoxLeft, nBoxBottom, nBoxRight AS NUMERIC INIT 0
   DATA   lBoxDraw

   CLASSDATA lRegistered AS LOGICAL
   CLASSDATA nForms AS NUMERIC INIT 0
   CLASSDATA oInspector   // reference to the object inspector window

   METHOD New( cName ) CONSTRUCTOR

   METHOD Add( oControl )

   METHOD ControlsTabOrder()

   METHOD DlgToCls()
   METHOD DlgToPRG()
   METHOD DlgToRC()
   METHOD DrawBox() INLINE RectDotted( ::hWnd, ::nBoxTop,;
                    ::nBoxLeft, ::nBoxBottom, ::nBoxRight )

   METHOD GetItems()
   METHOD GetItemsList()
   METHOD GetNewPrompt( cCtrlClass )
   METHOD GetNewVarName()
   METHOD GotFocus( hWndFocus )

   METHOD GridSettings()

   METHOD LButtonDown( nRow, nCol, nFlags )
   METHOD LButtonUp( nRow, nCol, nFlags )
   METHOD MouseMove( nRow, nCol, nFlags )

   METHOD LoadFile( cFileName )

   METHOD RButtonDown( nRow, nCol, nFlags )

   METHOD _SetCursor( cCrsName )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( cName ) CLASS TDesigner

   local oIco
   local aGrid

   #ifdef __XPP__
      DEFAULT ::nForms := 0

      ::lPaintGrid = .t.
   #endif

   ::nForms++

   DEFAULT cName := "form" + ;
      Str( ::nForms, If( ::nForms < 10, 1,;
      If( ::nForms < 100, 2, 3 ) ) )

   ::nStyle = nOr( WS_CLIPCHILDREN, WS_CAPTION, WS_SYSMENU,;
                   WS_MINIMIZEBOX, WS_MAXIMIZEBOX, WS_VSCROLL,;
                   WS_HSCROLL, WS_THICKFRAME )
   ::aControls  = {}
   ::cVarName   = "oForm" + Str( ::nForms, If( ::nForms < 10, 1, 2 ) )
   ::cFileName  = Lower( SubStr( ::cVarName, 2 ) ) + ".ffm"
   ::cPrgName   = Lower( SubStr( ::cVarName, 2 ) ) + ".prg"
   ::cClassName = "TForm" + Str( ::nForms, If( ::nForms < 10, 1,;
                                 If( ::nForms < 100, 2, 3 ) ) )
   ::lBoxDraw   = .f.

   #ifdef __XPP__
      DEFAULT ::lRegistered := .f.
   #endif

   ::Register()
   ::hWnd = CreateWindow( ::ClassName(),;
               StrCapFirst( cFileNoExt( cFileNoPath( cName ) ) ), ::nStyle,;
               311, 191, 544, 375 )
   ::Link()
   ::Default()

   DEFINE SCROLLBAR ::oVScroll VERTICAL OF Self RANGE 0, 0
   DEFINE SCROLLBAR ::oHScroll HORIZONTAL OF Self RANGE 0, 0

   DEFINE ICON oIco RESOURCE "FiveWin"
   ::SetIcon( oIco )
   ::SetColor( CLR_BLACK, GetSysColor( CLR_BTNFACE ) )

   DEFINE FONT ::oFont NAME GetSysFont() SIZE 0, -12

   ::bMenuSelect = { | oMItem | If( oMItem != nil,;
                     ::oInspector:SetObject( oMItem ),) }

   aGrid = GetGridSize()
   ::nGridX = aGrid[ 1 ]
   ::nGridY = aGrid[ 2 ]
   ::bPainted = { | hDC, cPS | If( ::lPaintGrid,;
                                   DrawGrid( ::hWnd, hDC, cPS, ::nGridX, ::nGridY ),) }

   SetWndDefault( Self )

   ::bValid  := { || ::Hide(), .f. }
   ::lDesign := .t.

   ::oInspector = oIDE:oInspector

   ::bLostFocus = { || oIDE:oWndActive := Self }
   ::bResized   = { || If( GetFocus() == ::hWnd, ::oInspector:SetForm( Self ),) }
   ::bMoved     = { || ::oInspector:SetForm( Self ) }

return Self

//----------------------------------------------------------------------------//

METHOD Add( oControl ) CLASS TDesigner

   ::oInspector:oCombo:Add( oControl:cVarName + " AS " + ;
                          oControl:ClassName() )

return nil

//----------------------------------------------------------------------------//

METHOD DlgToCls() CLASS TDesigner

   local cPrgName := cGetFile( "*.prg",;
                     "Save Dialog as a Class in PRG",,, .t. )

   MsgInfo( cPrgName )

return nil

//----------------------------------------------------------------------------//

METHOD DlgToPRG() CLASS TDesigner

   local cPrgName := cGetFile( "*.prg", "Save Dialog as PRG",,, .t. )

   if File( cPrgName )
      if ! MsgYesNo( cPrgName + CRLF + ;
                     "This file already exists" + CRLF + ;
                     " Do you want to overwrite it ?" )
         return nil
      endif
   endif

   MemoWrit( cPrgName, ::cGenPrg() )
   MsgInfo( "Done!" )

return nil

//----------------------------------------------------------------------------//

METHOD DlgToRC() CLASS TDesigner

   local cRcName := cGetFile( "*.rc", "Save Dialog as RC",,, .t. )
   local cRc     := ::cVarName + " DIALOG " + LTrim( Str( ::nLeft ) ) + ", " + ;
                    LTrim( Str( ::nTop ) ) + ", " + LTrim( Str( ::nWidth ) ) + ", " + ;
                    LTrim( Str( ::nHeight ) ) + CRLF + ;
                    "STYLE DS_MODALFRAME | WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU"

   if File( cRcName )
      if ! MsgYesNo( "Do you want to overwrite it ?",;
                     cRcName + " file already exists!" )
         return nil
      endif
   endif

   MsgInfo( cRc )

return nil

//----------------------------------------------------------------------------//

METHOD GetItems() CLASS TDesigner

   local aItems := ::GetItemsList()

   AAdd( aItems, nil )

   if Len( aItems ) > 1
      AIns( aItems, 1 )
   endif

   aItems[ 1 ] = ::cVarName + " AS " + ::cClassName

return aItems

//----------------------------------------------------------------------------//

METHOD GetNewPrompt( cCtrlClass ) CLASS TDesigner

   local nCtrl := 1, n

   cCtrlClass = Upper( cCtrlClass )

   for n = 1 to Len( ::aControls )
      if Upper( ::aControls[ n ]:ClassName() ) == cCtrlClass
         nCtrl++
      endif
   next

return "&" + StrCapFirst( SubStr( cCtrlClass, 2 ) + Str( nCtrl, If( nCtrl < 10, 1, 2 ) ) )

//----------------------------------------------------------------------------//

METHOD GetNewVarName() CLASS TDesigner

   local cCtrlClass := ATail( ::aControls ):ClassName()
   local nCtrl      := 0
   local n

   for n = 1 to Len( ::aControls )
      if ::aControls[ n ]:ClassName() == cCtrlClass
         nCtrl++
      endif
   next

return "o" + StrCapFirst( SubStr( cCtrlClass, 2 ) + ;
       Str( nCtrl, If( nCtrl < 10, 1, 2 ) ) )

//----------------------------------------------------------------------------//

METHOD GotFocus( hWndFocus ) CLASS TDesigner

   if ! Empty( ::aControls )
      ::aControls[ 1 ]:HideDots()
   endif

   ::oInspector:SetForm( Self )

return nil

//----------------------------------------------------------------------------//

METHOD LButtonDown( nRow, nCol, nFlags ) CLASS TDesigner

   SetFocus( ::hWnd )
   ::oInspector:SetForm( Self )

   if ! oIDE:oActiveForm == Self
      oIDE:oActiveForm = Self
      oIDE:oWndCode:oActiveEdit = Self:oEdit
   endif

   if ::bLClicked != nil
      Eval( ::bLClicked, nRow, nCol, nFlags )
   else
      ::lBoxDraw   = .t.
      ::nBoxTop    = nRow
      ::nBoxLeft   = nCol
      ::nBoxBottom = nRow
      ::nBoxRight  = nCol
      ::Capture()
      ::DrawBox()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD MouseMove( nRow, nCol, nFlags ) CLASS TDesigner

   if ::lBoxDraw
      ::DrawBox()
      ::nBoxBottom = nRow
      ::nBoxRight  = nCol
      ::DrawBox()
   endif

return Super:MouseMove( nRow, nCol, nFlags )

//----------------------------------------------------------------------------//

METHOD LButtonUp( nRow, nCol, nFlags ) CLASS TDesigner

   local cCtrl, cTemp := Space( 10 ), lTemp := .f., oCtrl

   if ::lBoxDraw
      ::DrawBox()
      ::lBoxDraw = .f.
      ReleaseCapture()

      if oIde:cToolControl != nil
         cCtrl = oIde:cToolControl

         if ::nBoxBottom == ::nBoxTop
            ::nBoxBottom = ::nBoxTop + 50
         endif   
         if ::nBoxRight == ::nBoxLeft
            ::nBoxRight = ::nBoxLeft + 100
         endif   

         do case
            case cCtrl == "Label"
                 @ ::nBoxTop, ::nBoxLeft SAY oCtrl PROMPT ::GetNewPrompt( "TSay" ) ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN

            case cCtrl == "Button"
                 @ ::nBoxTop, ::nBoxLeft BUTTON oCtrl PROMPT ::GetNewPrompt( "TButton" ) ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN

            case cCtrl == "CheckBox"
                 @ ::nBoxTop, ::nBoxLeft CHECKBOX oCtrl VAR lTemp PROMPT ::GetNewPrompt( "TCheckBox" ) ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN

            case cCtrl == "Get"
                 @ ::nBoxTop, ::nBoxLeft GET oCtrl VAR cTemp ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN
                 oCtrl:Set3DLook()
                 oCtrl:VarPut( ::GetNewVarName() )
                 oCtrl:Refresh()

            case cCtrl == "ListBox"
                 @ ::nBoxTop, ::nBoxLeft LISTBOX oCtrl VAR cTemp ;
                    ITEMS { "one", "two", "three" } ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN
                 ATail( ::aControls ):Set3DLook()

            case cCtrl == "ComboBox"
                 @ ::nBoxTop, ::nBoxLeft COMBOBOX oCtrl VAR cTemp ;
                    ITEMS { "one", "two", "three" } ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN

            case cCtrl == "Horizontal scroll"
                 @ ::nBoxTop, ::nBoxLeft SCROLLBAR oCtrl ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN HORIZONTAL

            case cCtrl == "Vertical scroll"
                 @ ::nBoxTop, ::nBoxLeft SCROLLBAR oCtrl ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN VERTICAL

            case cCtrl == "TreeView"
                 @ ::nBoxTop, ::nBoxLeft TREEVIEW oCtrl ;
                    SIZE ::nBoxRight - ::nBoxLeft + 1, ::nBoxBottom - ::nBoxTop + 1 ;
                    PIXEL OF Self DESIGN
                 oCtrl:SetItems( { "One", "Two", "Three" } )
                 
            case cCtrl == "Pulldown menu"
                 MENU oCtrl
                    MENUITEM "One"
                    MENU
                       MENUITEM "First"
                    ENDMENU

                    MENUITEM "Two"
                    MENU
                       MENUITEM "Second"
                    ENDMENU

                    MENUITEM "Three"
                    MENU
                       MENUITEM "Third"
                    ENDMENU
                 ENDMENU         
                 ::SetMenu( oCtrl )

         endcase

         if ! Empty( oCtrl ) .and. ! cCtrl $ "Pulldown menu"
            oCtrl = ATail( ::aControls )  
            oCtrl:cVarName = ::GetNewVarName()
            oCtrl:bGotFocus = { || ::oInspector:SetObject( oCtrl ) }
            oCtrl:SetFocus()
            ::Add( oCtrl )
            oIde:oWndCode:AddData( oCtrl:cVarName )
         endif   
         oIde:cToolControl = nil
      endif

   endif

return nil

//----------------------------------------------------------------------------//

METHOD LoadFile( cFileName ) CLASS TDesigner

   local cText := MemoRead( cFileName )
   local n, nLines := MLCount( cText ), cLine, cToken
   local oCtrlClass, oCtrl

   for n = 1 to nLines
      cLine = MemoLine( cText, 80, n )
      cToken = Alltrim( StrToken( cLine, 1 ) )
      do case
         case cToken == "OBJECT"
              if Left( StrToken( cLine, 2 ), 2 ) != "::"
                 ::cVarName = StrToken( cLine, 2 )
              else
                 oCtrlClass = &( StrToken( cLine, 4 ) + "()" )
                 AAdd( ::aControls, oCtrl := OSend( oCtrlClass, "New" ) )
                 oCtrl:cVarName = SubStr( StrToken( cLine, 2 ), 3 )
                 OSend( Self, "_" + oCtrl:cVarName, oCtrl )
                 oCtrl:lDrag = .t.
                 #ifndef __XPP__
                    Self = oCtrl
                 #endif
              endif

         case Left( cToken, 2 ) == "::"
              OSend( Self, "_" + SubStr( cToken, 3 ), &( StrToken( cLine, 2, "=" ) ) )

         case cToken == "ENDOBJECT"
              #ifndef __XPP__
                 Self = QSelf()
              #endif
      endcase
   next

   if Len( ::aControls ) > 0
      ::aControls[ 1 ]:lDrag = .f.
      ::aControls[ 1 ]:CheckDots()
      ::aControls[ 1 ]:lDrag = .t.
      ::aControls[ 1 ]:ShowDots()
      ::aControls[ 1 ]:SetFocus()
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GetItemsList() CLASS TDesigner

   local aControls := {}, n

   for n = 1 to Len( ::aControls )
      AAdd( aControls, ::aControls[ n ]:cVarName + " AS " + ;
            ::aControls[ n ]:ClassName() )
   next

return aControls

//----------------------------------------------------------------------------//

METHOD RButtonDown( nRow, nCol, nFlags ) CLASS TDesigner

   local oPopup
   local bMenuSelect := ::bMenuSelect  // save it

   ::bMenuSelect = nil                 // Reset it

   MENU oPopup POPUP 2007
      MENUITEM "&View form as..." RESOURCE "source" 
      MENU
         MENUITEM "Text" + Chr( 9 ) + ".TXT" ;
            ACTION oIDE:oWndCode:AddPage( Lower( cFileNoExt( ::cFileName ) ) + ".txt", ::SaveToText() ),;
                   oIDE:oWndCode:SetFocus()
         
         MENUITEM "Resource" + Chr( 9 ) + ".RC"
         MENUITEM "FiveWin source" + Chr( 9 ) + ".PRG"
      ENDMENU
                
      SEPARATOR
      
      MENUITEM "&Lock controls" RESOURCE "lock" ;
         ACTION AEval( ::aControls, { | o | o:lDrag := .F. } ), ::aControls[ 1 ]:HideDots() ;
         WHEN Len( ::aControls ) > 0

      MENUITEM "&Unlock controls" ;
         ACTION AEval( ::aControls, { | o | o:lDrag := .T. } ), ::aControls[ 1 ]:ShowDots() ;
         WHEN Len( ::aControls ) > 0
      
      SEPARATOR
      
      MENUITEM "&Grid Settings..." RESOURCE "grid" ACTION ::GridSettings()
      
      SEPARATOR
      
      MENUITEM "&Controls Creation/Tab Order..."  ACTION ::ControlsTabOrder() ;
         WHEN Len( ::aControls ) > 1
      
   ENDMENU

   ACTIVATE POPUP oPopup AT nRow, nCol OF Self

   ::bMenuSelect = bMenuSelect         // restore it

return nil

//----------------------------------------------------------------------------//

static function DlgTest( oWnd, cDlgName )

   local oDlg, aPoint := { 0, 0 }

   oWnd:CoorsUpdate()
   ClientToScreen( oWnd:hWnd, aPoint )

   DEFINE DIALOG oDlg OF oWnd ;
      TITLE cDlgName ;
      FROM oWnd:nTop, oWnd:nLeft TO ;
           oWnd:nBottom - 30, oWnd:nRight - 12 PIXEL

   @ 2, 2 BUTTON "&Ok" OF oDlg

   ACTIVATE DIALOG oDlg ;
      ON INIT oDlg:Move( aPoint[ 1 ] - GetSysMetrics( 4 ) - ;
                         GetSysMetrics( 33 ) + 1,;
                         aPoint[ 2 ] - GetSysMetrics( 32 ) )

return nil

//----------------------------------------------------------------------------//

METHOD _SetCursor( cCrsName ) CLASS TDesigner

   local oCursor

   if ! Empty( cCrsName )
      DEFINE CURSOR oCursor RESOURCE cCrsName
   endif

   if ::oCursor != nil
      ::oCursor:End()
      ::oCursor = nil
   endif

   if oCursor != nil
      ::oCursor = oCursor
   endif

return nil

//----------------------------------------------------------------------------//

METHOD GridSettings() CLASS TDesigner

   local oDlg
   local nOldVert := ::nGridX, nOldHorz := ::nGridY, lOldGrid := ::lPaintGrid
   local nVert := ::nGridX, nHorz := ::nGridY
   local lGrid := ::lPaintGrid
   local oThis := Self

   DEFINE DIALOG oDlg RESOURCE "GridSettings"

   REDEFINE GET nVert ID ID_VERTICAL ;
      VALID ( ::nGridX := nVert, ::Refresh(), .t. )

   REDEFINE GET nHorz ID ID_HORIZONTAL ;
      VALID ( ::nGridY := nHorz, ::Refresh(), .t. )

   REDEFINE CHECKBOX lGrid ID ID_ENABLE ;
      ON CHANGE ( ::lPaintGrid := lGrid, ::Refresh() )

   REDEFINE BUTTON ID ID_OK ;
      ACTION ( ::nGridX := nVert, ::nGridY := nHorz, ::lPaintGrid := lGrid,;
               ::Refresh(), oDlg:End() )

   REDEFINE BUTTON ID ID_CANCEL ;
      ACTION ( ::nGridX := nOldVert, ::nGridY := nOldHorz,;
               ::lPaintGrid := lOldGrid, ::Refresh(), oDlg:End() )

   ACTIVATE DIALOG oDlg CENTERED

return nil

//----------------------------------------------------------------------------//

METHOD ControlsTabOrder() CLASS TDesigner

   local oDlg, oLbx, cControl

   DEFINE DIALOG oDlg RESOURCE "ControlsTabOrder"

   REDEFINE LISTBOX oLbx VAR cControl ITEMS ::GetItemsList() ;
      ID ID_CONTROLS OF oDlg

   REDEFINE BUTTON ID ID_UP   OF oDlg ACTION oLbx:SwapUp()
   REDEFINE BUTTON ID ID_DOWN OF oDlg ACTION oLbx:SwapDown()

   ACTIVATE DIALOG oDlg CENTERED

return nil

//----------------------------------------------------------------------------//
