// FiveWin Visual IDE
// (c) FiveTech 1993-2008

#include "FiveWin.ch"
#include "Visual.ch"
#include "splitter.Ch"

memvar oIDE

#define VFW_VERSION  "Visual FiveWin for Harbour"

#define SIZE_RESTORED   0
#define SIZE_MINIMIZED  1
#define SIZE_MAXIMIZED  2

//----------------------------------------------------------------------------//

function Main( cFileName )

   local oWnd
   local hRichDLL := LoadLibrary( "riched20.dll" )  // Richedit support

   public oIDE

   // To trace calls for FWH
   // __TRACEPRGCALLS( .t. )
   // HB_TRACELEVEL( 5 )
   // hb_tracestate( 1 )

   MsgLogo( "FW" )           // "FW" is the name of a resource bitmap

   SetResDebug( .t. )

   SET HELPFILE TO "docs\Visual.hlp"

   oIDE = TVisualIDE():New()

   oIDE:Activate( cFileName )

   // CheckRes()
   FreeLibrary( hRichDLL )

return nil

//----------------------------------------------------------------------------//

CLASS TVisualIDE

   DATA   oWndMain           // Main window of the application
   DATA   oInspector         // Object Inspector window
   DATA   oWndCode           // Source code editing window
   DATA   oToolBar           // ToolBar window
   DATA   oProject           // Project window

   DATA   oActiveForm        // Current focused edited form

   DATA   oMruDbfs
   DATA   oMruPrgs           // MRU-Programs
   DATA   oMruPrjs           // MRU-Projects
   DATA   oMruRes

   DATA   oItForms           // MenuItem that holds all the active edited forms

   DATA   cArguments         // Application Execution arguments

   DATA   oHand              // Cursor Hand

   DATA   oWndActive         // Form or Source code editor

   DATA   oSplitter          // middle Splitter

   DATA   oToolBar           // right side toolbar
   DATA   nToolBar           // Currently selected toolbar category

   DATA   cToolControl       // temporarely holds the tool control name (its tooltip)
                             // to place on the form

   METHOD New() CONSTRUCTOR

   METHOD Activate( cFileName )

   METHOD BuildBars()
   METHOD BuildMenu()

   METHOD Close()

   METHOD End() INLINE ::oWndMain:End()

   METHOD LoadForm( cFileName )

   METHOD NewForm()
   METHOD NewProgram()

   METHOD Open()
   METHOD OpenForm( cFormFile )
   METHOD OpenProgram( cPrgName )   
   METHOD OpenFiles( aFiles )

   METHOD Arguments()

   METHOD MainReSize( nMode )

   METHOD Save()
   METHOD SaveAs()

   METHOD SelToolBarBtn( oBtn )   
   METHOD SetToolBar( nToolBar )  // Set the toolbar group of buttons

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New() CLASS TVisualIDE

   local oFont, oTabs

   ::cArguments = ""

   oIDE = Self

   DEFINE FONT oFont NAME "Ms Sans Serif" SIZE 0, -8

   DEFINE WINDOW ::oWndMain TITLE "Visual FiveWin" ;
      MENU ::BuildMenu() FROM 0, 0 TO 7, 120 ;
      ICON "FiveWin"

   ::oWndMain:aMinMaxInfo = { GetSysMetrics() + 7,;
                              If( ! IsAppThemed(), 103, 110 ) * DispFontPerSize(),;
                              -4, -4, 400, 118,;
                              GetSysMetrics() + 7,;
                              If( ! IsAppThemed(), 103, 110 ) * DispFontPerSize() }
   ::oWndMain:bReSized    = { | nMode, nWidth, nHeight | ;
                              ::MainResize( nMode ) }

   DEFINE CURSOR ::oHand HAND

   ::BuildBars()

   ::oInspector  = TInspector():New()
   ::oWndCode    = TWndCode():New()
   ::oProject    = TWndPrj():New()
   ::oWndMain:SetText( VFW_VERSION + " - " + ::oProject:cName )

   ::oProject:Hide()
   ::NewForm()

return Self

//----------------------------------------------------------------------------//

METHOD Activate( cFileName ) CLASS TVisualIDE

   local cExt, oThis := Self

   if ! Empty( cFileName )
      ::OpenFiles( { cFileName } )
   endif

   ACTIVATE WINDOW ::oWndMain MAXIMIZED ;
      VALID If( MsgYesNo( "Want to end ?" ), ( ::Save(), .t. ), .f. ) ;
      ON DROPFILES ::OpenFiles( aFiles ) ;
      ON RESIZE ::oSplitter:AdjClient() ;
      ON INIT ( oThis:oActiveForm:SetFocus() )

   ::oHand:End() // Release cursor hand

return nil

//----------------------------------------------------------------------------//

METHOD BuildBars() CLASS TVisualIDE

   local oPanel1 := TPanel():New( 0, 0, 0, 363, ::oWndMain )
   local oPanel2 := TPanel():New( 0, 368, 0, 200, ::oWndMain )
   local oBar1, oBar2, oTabs, oBtnForms, n

   DEFINE BUTTONBAR oBar1 OF oPanel1 2007 SIZE 30, 30
   oBar1:bRClicked = { || nil }
   oBar1:bLClicked = { || nil }

   DEFINE BUTTON RESOURCE "Exit" NOBORDER OF oBar1 ;
      ACTION ::oWndMain:End() TOOLTIP "Exit" ;
      MESSAGE "Exit from IDE"

   DEFINE BUTTON RESOURCE "Open" NOBORDER OF oBar1 ;
      TOOLTIP "Open" ;
      ACTION  ::Open()

   DEFINE BUTTON RESOURCE "Save" NOBORDER OF oBar1 ;
      TOOLTIP "Save" ;
      MESSAGE "Save current project to disk" ;
      ACTION  ::Save()

   DEFINE BUTTON RESOURCE "newform" NOBORDER OF oBar1 GROUP ;
      TOOLTIP "New form" ;
      MESSAGE "Creates a new form" ACTION ::NewForm()

   DEFINE BUTTON RESOURCE "newprg" NOBORDER OF oBar1 ;
      TOOLTIP "New program" ;
      MESSAGE "Creates a new prg" ACTION ::NewProgram()

   DEFINE BUTTON RESOURCE "Cut" NOBORDER OF oBar1 GROUP ;
      TOOLTIP "Cut" ;
      MESSAGE "Cut to Clipboard" ACTION MsgInfo( "Cut" )

   DEFINE BUTTON RESOURCE "Copy" NOBORDER OF oBar1 ;
      TOOLTIP "Copy" ;
      MESSAGE "Copy to Clipboard" ACTION ::oWndMain:Copy()

   DEFINE BUTTON RESOURCE "Paste" NOBORDER OF oBar1 ;
      TOOLTIP "Paste" ACTION ::oWndMain:Paste()

   DEFINE BUTTON RESOURCE "Delete" NOBORDER OF oBar1 ;
      TOOLTIP "Delete" ACTION ::oWndMain:Delete()

   DEFINE BUTTON RESOURCE "UnDo" NOBORDER OF oBar1 GROUP ;
      TOOLTIP "UnDo" ACTION ::UnDo()

   DEFINE BUTTON RESOURCE "ReDo" NOBORDER OF oBar1 ;
      TOOLTIP "ReDo" ACTION ::ReDo()

   DEFINE BUTTONBAR oBar2 OF oPanel1 2007 SIZE 30, 30 BOTTOM
   oBar2:bLClicked = { || nil }
   oBar2:bRClicked = { || nil }
   oPanel1:oTop = oBar1
   oPanel1:oBottom = oBar2

   DEFINE BUTTON RESOURCE "project" NOBORDER OF oBar2 ;
      TOOLTIP "Project manager" ;
      ACTION ( ::oProject:Show(), ::oProject:Restore(), ::oProject:SetFocus() )

   DEFINE BUTTON oBtnForms RESOURCE "Form" NOBORDER OF oBar2 GROUP ;
      TOOLTIP "Forms" ACTION oBtnForms:ShowPopup() MENU ::oItForms:bAction

   DEFINE BUTTON RESOURCE "Source" NOBORDER OF oBar2 ;
      TOOLTIP "Source editor" ACTION ::oWndCode:SetFocus()

   DEFINE BUTTON RESOURCE "Setup" NOBORDER OF oBar2 GROUP ;
      TOOLTIP "Project setup" ACTION ::oProject:Setup()

   DEFINE BUTTON RESOURCE "Build" NOBORDER OF oBar2 ;
      TOOLTIP "Build" ACTION ::oProject:Build()

   DEFINE BUTTON RESOURCE "Exec" NOBORDER OF oBar2 ;
      TOOLTIP "Run" ACTION ::oProject:Run( ::cArguments )

   DEFINE BUTTON RESOURCE "Debug" NOBORDER OF oBar2 TOOLTIP "Debug" ;
      ACTION ::oProject:Debug( ::cArguments )

   DEFINE BUTTON RESOURCE "About" NOBORDER OF oBar2 GROUP ;
      TOOLTIP "Info" ;
      ACTION MsgLogo( "fw" )

   AEval( oBar1:aControls, { | o | o:oCursor := ::oHand } )
   AEval( oBar2:aControls, { | o | o:oCursor := ::oHand } )

   DEFINE BUTTONBAR ::oToolBar OF oPanel2 SIZE 45, 45 2007
   ::oToolBar:bLClicked = { || nil }
   ::oToolBar:bRClicked = { || nil }
   
   DEFINE BUTTON OF ::oToolBar RESOURCE "arrow" ;
      TOOLTIP "Pointer" ACTION ::SelToolBarBtn( this ) PRESSED   

   for n = 1 to 13
      DEFINE BUTTON OF ::oToolBar ACTION ::SelToolBarBtn( this )

      ::oToolBar:aControls[ n + 1 ]:bWhen = GenNonEmptyToolTipBlock( ::oToolBar:aControls[ n + 1 ] )
   next   

   oPanel2:oTop = ::oToolBar
   
   ::SetToolBar( 1 )

   @ 8, 0 TABS oTabs PROMPTS "&Standard", "&Additional", "3rd &party", "&Database",;
        "D&ialogs", "In&ternet", "Active&X" OF oPanel2 ;
        ACTION ::SetToolBar( nOption )

   oPanel2:oBottom = oTabs

   @ 0, 363 SPLITTER ::oSplitter ;
            VERTICAL ;
            PREVIOUS CONTROLS oPanel1 ;
            HINDS CONTROLS oPanel2 ;
            LEFT MARGIN 90 ;
            RIGHT MARGIN 180 ;
            SIZE 4, nil  PIXEL ;
            OF ::oWndMain ;
            _3DLOOK ;
            UPDATE

return nil

//----------------------------------------------------------------------------//

METHOD BuildMenu() CLASS TVisualIDE

   local oMenu

   MENU oMenu 2007
      MENUITEM "&Projects"
      MENU
        MENUITEM "&New..."  MESSAGE "Creates a new project" RESOURCE "NewProj"
        MENUITEM "&Open..." MESSAGE "Open an existing project" RESOURCE "Open"
        MENU
           MENUITEM "&Search... " RESOURCE "search" ;
              ACTION cGetFile( "VFWH Project file (*.fpr) | *.fpr |",;
                               "Select a VFWH Project to open" )
           MRU ::oMruPrjs INI ".\Visual.ini" SECTION "MRU-Projects" ;
              MESSAGE "Open this project"
              // ACTION PrjOpen( cMruItem, ::oMruPrjs )
        ENDMENU

        MENUITEM "&Save..."      ACTION ::Save() RESOURCE "Save"
        MENUITEM "Save &as..."   ACTION ::SaveAs()
        MENUITEM "Close Project" ACTION ::Close()
        SEPARATOR

        MENUITEM "New &Form..."     ACTION ::NewForm()     RESOURCE "newform"
        MENUITEM "New &Program..."  ACTION ::NewProgram()  RESOURCE "newprg"
        MENUITEM "Open F&orm..."    ACTION ::OpenForm()    RESOURCE "form"
        MENUITEM "Open P&rogram..." ACTION ::OpenProgram() RESOURCE "source"
        SEPARATOR

        MENUITEM "&Setup..."     MESSAGE "Setup the current project" ;
           ACTION ::oProject:Setup() RESOURCE "Setup"
        MENUITEM "&Build..."     MESSAGE "Builds the current project" ;
           ACTION ::oProject:Build() RESOURCE "Build"
        MENUITEM "&Run..."       MESSAGE "Executes the current project" ;
           ACTION ::oProject:Run( ::cArguments ) RESOURCE "Exec"
        MENUITEM "&Debug..."       MESSAGE "Debugs the current project" ;
           ACTION ::oProject:Debug( ::cArguments ) RESOURCE "Debug"
        MENUITEM "&Arguments..." MESSAGE "Current project run arguments" ;
           ACTION ::Arguments()

        SEPARATOR
        MENUITEM "&Print..." RESOURCE "Print"
        MENUITEM "Printer &Setup..." ACTION PrinterSetup()

        SEPARATOR
        MENUITEM "E&xit..." ACTION ::oWndMain:End() ;
           MESSAGE "Exit VFWH IDE" RESOURCE "exit"
      ENDMENU

      oMenu:AddEdit()

      MENUITEM "&Databases"
      MENU
         MENUITEM "&Use DataBase..." ACTION DbfOpen(,, ::oMruDbfs ) ;
            MESSAGE "Open a DataBase"

         MENUITEM "&Create DataBase..."  ACTION DbfNew( ::oMruDbfs ) ;
            MESSAGE "Create a new DataBase"

         MRU ::oMruDbfs INI ".\Visual.ini" SECTION "MRU-DataBases" ;
            MESSAGE "Browse this DataBase" ;
            ACTION DbfOpen( cMruItem,, ::oMruDbfs )

      ENDMENU

      MenuAddTools( ".\visual.ini" )

      MENUITEM "&Views"
      MENU
         MENUITEM "&Object Inspector..." RESOURCE "properties" ;
            ACTION ( ::oInspector:Show(), ::oInspector:SetFocus() ) ;
            MESSAGE "Shows the Object Inspector"

         MENUITEM "&Project Manager..." RESOURCE "project" ;
            ACTION ( ::oProject:Show(), ::oProject:Restore(),;
                     ::oProject:SetFocus() ) ;
            MESSAGE "Shows the Project Manager"

         MENUITEM "&Source code editor..." RESOURCE "source" ;
            ACTION ( ::oWndCode:Show(), ::oWndCode:SetFocus() ) ;
            MESSAGE "Shows the source code editor"

         SEPARATOR

         MENUITEM ::oItForms PROMPT "&Forms"
         MENU
         ENDMENU
      ENDMENU

      oMenu:AddHelp( { || MsgLogo( "fw" ) } )

   ENDMENU

return oMenu

//----------------------------------------------------------------------------//

METHOD Close() CLASS TVisualIDE

   if ::oProject != nil
      ::oInspector:Close()
      ::oInspector = nil
      ::oWndCode:Close()
      ::oWndCode = nil
      ::oProject:Close()
      ::oProject = nil
      ::oWndMain:SetText( VFW_VERSION )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD LoadForm( cFileName ) CLASS TVisualIde

   local oForm := TDesigner():New(), oItem

   oForm:LoadFile( cFileName )

   ::oItForms:bAction:Add( oItem := TMenuItem():New( oForm:cVarName,,,,;
                           { || oForm:Show(), oForm:SetFocus() },, "form" ) )
   ::oActiveForm = oForm
   ::oInspector:SetForm( oForm )
   
   oForm:Show()
   oForm:SetFocus()

return nil

//----------------------------------------------------------------------------//

METHOD NewForm() CLASS TVisualIDE

   local oForm := ::oProject:NewForm()

   ::oItForms:bAction:Add( TMenuItem():New( oForm:cVarName,,,,;
                           { || oForm:Show(), oForm:SetFocus() },, "form" ) )
   ::oActiveForm = oForm
   ::oInspector:SetForm( oForm )
   oForm:Show()
   oForm:SetFocus()

return nil

//----------------------------------------------------------------------------//

METHOD NewProgram() CLASS TVisualIDE

   local oForm := ::oWndCode:AddPage( "noname.prg",;
      '#include "FiveWin.ch"' + CRLF + CRLF + ;
      "//------------------------------------------------------" + ;
      "---------------------//" + CRLF + CRLF + ;
      "function Noname()" + CRLF + CRLF + CRLF + CRLF + "return nil" + CRLF + CRLF + ;
      "//------------------------------------------------------" + ;
      "---------------------//" )

   AAdd( ::oProject:aPrograms, { "noname.prg", "" } )
   ::oProject:oBrw:Refresh()

   ::oWndCode:SetFocus()

return nil

//----------------------------------------------------------------------------//

METHOD Open() CLASS TVisualIDE

   local oIni, n := 1, cForm
   local cFileName := cGetFile( "VFWH Project file (*.fpr) | *.fpr |" + ;
                                "Program file (*.prg) | *.prg |" + ;
                                "Header file (*.ch) | *.ch |" + ;
                                "VFWH form (*.frm) | *.frm |" + ;
                                "Open as ASCII file (*.*) | *.* |",;
                                "Select a file to open" )
   local nFileType := nGetFileFilter()

   if Empty( cFileName )
      return nil
   endif

   if ! File( cFileName )
      MsgAlert( "File " + cFileName + " not found!" )
      return nil
   endif

   do case
      case nFileType == 1 // Project
           ::Save()
           ::Close()

           ::oInspector = TInspector():New()
           ::oProject   = TWndPrj():New( cFileName )
           ::oWndCode   = TWndCode():New()

           oIni = TIni():New( cFileName )

           if ! Empty( cForm := oIni:Get( "FORMS", AllTrim( Str( n++ ) ) ) )
              ::OpenForm( cForm )
           endif

           while ! Empty( cForm := oIni:Get( "FORMS", AllTrim( Str( n++ ) ) ) )
              AAdd( ::oProject:aForms, cForm )
           end

           ::oWndMain:SetText( VFW_VERSION + " - " + cFileName )

      case nFileType == 2 .or. nFileType == 3 .or. ;
           nFileType == 5 // Program, header file or ASCII file
           ::oWndCode:AddPage( cFileName, MemoRead( cFileName ) )

      case nFileType == 4 // Form
           ::LoadForm( cFileName )

   endcase

return nil

//----------------------------------------------------------------------------//

METHOD OpenFiles( aFiles ) CLASS TVisualIDE

   local cExt, n

   ::oWndMain:Maximize()      // restore main window

   for n = 1 to Len( aFiles )
      cExt = Upper( Right( aFiles[ n ], 3 ) )

      do case
         case cExt == "PRG" .or. cExt == ".CH"
              PrgOpen( aFiles[ n ], ::oMruPrgs )

         case cExt == "DBF"
              DbfOpen( aFiles[ n ],, ::oMruDbfs )
      endcase
   next

return nil

//----------------------------------------------------------------------------//

METHOD OpenForm( cFrmName ) CLASS TVisualIDE

   local oForm

   DEFAULT cFrmName := cGetFile( "VFWH Form ( *.frm ) | *.frm",;
                                 "Select a Form",, "forms" )

   if Empty( cFrmName ) 
      return nil
   endif   
   
   if ! File( cFrmName )
      MsgInfo( "I can't find that file!" )
      return nil
   endif

   oForm = ::oProject:OpenForm( cFrmName )
   ::oActiveForm = oForm
   ::oInspector:SetForm( oForm )

   ::oItForms:bAction:Add( TMenuItem():New( oForm:cVarName,,,,;
                           { || oForm:Show(), oForm:SetFocus() } ) )

return nil

//----------------------------------------------------------------------------//

METHOD OpenProgram( cPrgName ) CLASS TVisualIDE

   DEFAULT cPrgName := cGetFile( "Program( *.prg ) | *.prg",;
                                 "Select a PRG, CH or C", "*.prg", "programs" )

   if Empty( cPrgName )
      return nil
   endif   

   if ! File( cPrgName )
      MsgInfo( "I can't find that file!" )
      return nil
   endif

   ::oWndCode:AddPage( cPrgName, MemoRead( cPrgName ) )

   AAdd( ::oProject:aPrograms, { cPrgName, "" } )
   ::oProject:oBrw:Refresh()

   ::oWndCode:SetFocus()

return nil

//----------------------------------------------------------------------------//

METHOD Arguments() CLASS TVisualIDE

   local oDlg
   local cArgs := PadR( ::cArguments, 128 )

   DEFINE DIALOG oDlg RESOURCE "Arguments"

   REDEFINE GET cArgs ID 110 OF oDlg

   REDEFINE BUTTON ID IDOK OF oDlg ;
      ACTION ::cArguments := RTrim( cArgs ), oDlg:End()

   ACTIVATE DIALOG oDlg CENTERED

return nil

//----------------------------------------------------------------------------//

METHOD MainResize( nMode ) CLASS TVisualIDE

   DEFAULT nMode := SIZE_RESTORED

   do case
      case nMode == SIZE_MINIMIZED
           ::oInspector:Hide()
           ::oWndCode:Hide()
           ::oToolBar:Hide()
           ::oProject:Hide()

      case nMode == SIZE_MAXIMIZED
           ::oInspector:Show()
           ::oWndCode:Show()
           ::oToolBar:Show()

      case nMode == SIZE_RESTORED
           ::oInspector:Show()
           ::oWndCode:Show()
           ::oToolBar:Show()
   endcase

return nil

//----------------------------------------------------------------------------//

METHOD Save() CLASS TVisualIDE

   if ::oProject != nil
      ::oProject:Save()
      ::oMruPrjs:Save( ::oProject:cName )
   endif

return nil

//----------------------------------------------------------------------------//

METHOD SaveAs() CLASS TVisualIDE

   ::oProject:cName = cGetFile( "*.fpr", "Select a project name" )
   ::oWndMain:SetText( VFW_VERSION + " - " + ::oProject:cName )
   ::Save()

return nil

//----------------------------------------------------------------------------//

METHOD SelToolBarBtn( oBtn ) CLASS TVisualIde

   local n

   for n = 1 to Len( ::oToolBar:aControls )
      if ! oBtn == ::oToolBar:aControls[ n ] .and. ;
         ::oToolBar:aControls[ n ]:lPressed
         ::oToolBar:aControls[ n ]:lPressed = .F.
         ::oToolBar:aControls[ n ]:Refresh()
      endif
   next

   if ! oBtn:lPressed
      oBtn:lPressed = .t.
      oBtn:Refresh()
      ::cToolControl = oBtn:cToolTip
   endif

return nil

//----------------------------------------------------------------------------//

METHOD SetToolBar( nToolBar ) CLASS TVisualIde

   local n, oBtn
   local aToolBars := { ;
      { { "label", "Label" }, { "button", "Button" }, { "edit", "Get" }, { "memo", "Get multiline" }, { "check", "CheckBox" }, { "radio", "RadioButton" }, { "listbox", "ListBox" }, { "combobox", "ComboBox" }, { "hscroll", "Horizontal scroll" }, { "vscroll", "Vertical scroll" }, { "group", "Group" }, { "progress", "Progress bar" }, { "slider", "Slider" } },;
      { { "menu", "Pulldown menu" }, { "bitmap", "Bitmap" }, { "image", "Image" }, { "folder", "Folder" }, { "treeview", "TreeView" }, { "tabs", "Tabs" }, { "meter", "Meter" }, { "msgbar", "MsgBar" }, { "icon", "Icon" }, { "panel", "Panel" }, { "btnbmp", "TBtnBmp" } },;
      {},;
      {},;
      {},;
      {},;
      { { "explorer", "Shell Explorer" } } }

   if ::nToolBar == nil .or. nToolBar != ::nToolBar
   
      ::nToolBar = nToolBar
   
      ::oToolBar:aControls[ 1 ]:lPressed = .T.
      ::oToolBar:aControls[ 1 ]:Refresh()
   
      for n = 2 to Len( ::oToolBar:aControls )
          oBtn = ::oToolBar:aControls[ n ]
          oBtn:FreeBitmaps()
          oBtn:cToolTip = nil
          oBtn:lPressed = .F.
          oBtn:Refresh()
      next     
      
      for n = 1 to Len( aToolBars[ nToolBar ] ) 
          oBtn = ::oToolBar:aControls[ n + 1 ]
          oBtn:LoadBitmaps( aToolBars[ nToolBar ][ n ][ 1 ] )
          oBtn:cToolTip = aToolBars[ nToolBar ][ n ][ 2 ]
      next
   
      ::oToolBar:AEvalWhen()
   endif

return nil

//----------------------------------------------------------------------------//

function GenNonEmptyToolTipBlock( oBtn )

return { || ! Empty( oBtn:cToolTip ) }

//----------------------------------------------------------------------------//